{% extends "pretixcontrol/event/base.html" %}
{% load i18n %}
{% block title %}{{ object.name }} :: {% trans "Product" %}{% endblock %}
{% block content %}
    {% if object.id %}
        <h1>{% trans "Modify product:" %} {{ object.name }}</h1>
        {% if object.has_variations %}
        <ul class="nav nav-pills">
            <li {% if "event.item" == url_name %}class="active"{% endif %}>
                <a href="{% url 'control:event.item' organizer=request.event.organizer.slug event=request.event.slug item=object.id %}">
                    {% trans "General information" %}
                </a>
            </li>
            <li {% if "event.item.variations" == url_name %}class="active"{% endif %}>
                <a href="{% url 'control:event.item.variations' organizer=request.event.organizer.slug event=request.event.slug item=object.id %}">
                    {% trans "Variations" %}
                </a>
            </li>
        </ul>
        {% endif %}
    {% else %}
        <h1>{% trans "Create product" %}</h1>
        <p>{% blocktrans trimmed %}
            You will be able to adjust further settings in the next step.
        {% endblocktrans %}</p>
    {% endif %}
    {% if object.id and not object.quotas.exists %}
        <div class="alert alert-warning">
        	{% blocktrans trimmed %}
                Please note that your product will <strong>not</strong> be available for sale until you have added your
                item to an existing or newly created quota.
	        {% endblocktrans %}
        </div>
    {% endif %}
	{% block inside %}
	{% endblock %}
{% endblock %}
